==> open-xchange-guard_2.4.0.orig.tar.bz2: open-xchange-guard-2.4.0/com.openexchange.guard.keys/src/com/openexchange/guard/keys/storage/keyCache/PopulateRSA.java <==
/*
 *
 *    OPEN-XCHANGE legal information
 *
 *    All intellectual property rights in the Software are protected by
 *    international copyright laws.
 *
 *
 *    In some countries OX, OX Open-Xchange, open xchange and OXtender
 *    as well as the corresponding Logos OX Open-Xchange and OX are registered
 *    trademarks of the OX Software GmbH group of companies.
 *    The use of the Logos is not covered by the GNU General Public License.
 *    Instead, you are allowed to use these Logos according to the terms and
 *    conditions of the Creative Commons License, Version 2.5, Attribution,
 *    Non-commercial, ShareAlike, and the interpretation of the term
 *    Non-commercial applicable to the aforementioned license is published
 *    on the web site http://www.open-xchange.com/EN/legal/index.html.
 *
 *    Please make sure that third-party modules and libraries are used
 *    according to their respective licenses.
 *
 *    Any modifications to this package must retain all copyright notices
 *    of the original copyright holder(s) for the original code used.
 *
 *    After any such modifications, the original and derivative code shall remain
 *    under the copyright of the copyright holder(s) and/or original author(s)per
 *    the Attribution and Assignment Agreement that can be located at
 *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
 *    given Attribution for the derivative code and a license granting use.
 *
 *     Copyright (C) 2016-2020 OX Software GmbH
 *     Mail: info@open-xchange.com
 *
 *
 *     This program is free software; you can redistribute it and/or modify it
 *     under the terms of the GNU General Public License, Version 2 as published
 *     by the Free Software Foundation.
 *
 *     This program is distributed in the hope that it will be useful, but
 *     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 *     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 *     for more details.
 *
 *     You should have received a copy of the GNU General Public License along
 *     with this program; if not, write to the Free Software Foundation, Inc., 59
 *     Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

package com.openexchange.guard.keys.storage.keyCache;

import java.io.IOException;
import java.io.StringWriter;
import java.math.BigInteger;
import java.security.KeyFactory;
import java.security.KeyPair;
import java.security.NoSuchAlgorithmException;
import java.security.NoSuchProviderException;
import java.security.PrivateKey;
import java.security.PublicKey;
import java.security.SecureRandom;
import java.security.spec.InvalidKeySpecException;
import java.security.spec.RSAPrivateCrtKeySpec;
import java.security.spec.RSAPublicKeySpec;
import java.sql.SQLException;
import org.bouncycastle.crypto.AsymmetricCipherKeyPair;
import org.bouncycastle.crypto.generators.RSAKeyPairGenerator;
import org.bouncycastle.crypto.params.RSAKeyGenerationParameters;
import org.bouncycastle.crypto.params.RSAKeyParameters;
import org.bouncycastle.crypto.params.RSAPrivateCrtKeyParameters;
import org.bouncycastle.openssl.jcajce.JcaPEMWriter;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import com.openexchange.exception.OXException;
import com.openexchange.guard.cipher.GuardCipherAlgorithm;
import com.openexchange.guard.cipher.GuardCipherFactoryService;
import com.openexchange.guard.cipher.GuardCipherService;
import com.openexchange.guard.common.java.Strings;
import com.openexchange.guard.configuration.GuardConfigurationService;
import com.openexchange.guard.configuration.GuardProperty;
import com.openexchange.guard.keys.GuardMasterKeyService;
import com.openexchange.guard.keys.osgi.Services;
import com.openexchange.guard.keys.storage.KeyCacheStorage;

public class PopulateRSA implements Runnable {

    private static Logger LOG = LoggerFactory.getLogger(PopulateRSA.class);

    @Override
    public void run() {
        try {
            KeyCacheStorage keyCacheStorage = Services.getService(KeyCacheStorage.class);
            GuardConfigurationService configService = Services.getService(GuardConfigurationService.class);
            int count = keyCacheStorage.getCount();
            int rsaCacheCount = configService.getIntProperty(GuardProperty.rsaCacheCount);
            if (count < rsaCacheCount) {
                while (count < rsaCacheCount) {
                    count++;
                    String newkey = genKeyPair();
                    storekey(newkey);
                }
            }
        } catch (Exception e) {
            LOG.error("Unable to access key_cache table.  Check database permissions", e);
        }
    }

    /**
     * Store RSA key PEM for future use if keyasc param is not empty. Encrypt with Config.rpass
     *
     * @param keyAsc the ascii armored to to save
     * @throws SQLException due an error
     */
    private void storekey(String keyAsc) throws SQLException {
        if (Strings.isEmpty(keyAsc)) {
            LOG.warn("Unable to store key as provided ascii armored version is empty!");
            return;
        }

        try {
            //encrypting the key for storing it in the DB
            GuardCipherService cipherService = Services.getService(GuardCipherFactoryService.class).getCipherService(GuardCipherAlgorithm.AES);
            GuardMasterKeyService masterKeyService = Services.getService(GuardMasterKeyService.class);
            keyAsc = cipherService.encrypt(keyAsc, masterKeyService.getMasterKey().getRC(), "key");

            KeyCacheStorage keyCacheStorage = Services.getService(KeyCacheStorage.class);
            keyCacheStorage.insert(keyAsc);
        } catch (Exception ex) {
            LOG.error("Error while storing key", ex);
        }
    }

    /**
     * Generate RSA keypair and return as PEM
     *
     * @return
     * @throws IOException
     * @throws NoSuchAlgorithmException
     * @throws InvalidKeySpecException
     * @throws NoSuchProviderException
     * @throws OXException
     */
    private String genKeyPair() throws IOException, NoSuchAlgorithmException, InvalidKeySpecException, NoSuchProviderException, OXException {
        GuardConfigurationService configService = Services.getService(GuardConfigurationService.class);
        RSAKeyPairGenerator kpgen = new RSAKeyPairGenerator();
        kpgen.init(new RSAKeyGenerationParameters(BigInteger.valueOf(0x10001), new SecureRandom(), configService.getIntProperty(GuardProperty.rsaKeyLength), configService.getIntProperty(GuardProperty.rsaCertainty)));
        AsymmetricCipherKeyPair kp = kpgen.generateKeyPair();
        RSAPrivateCrtKeyParameters rsapckp = (RSAPrivateCrtKeyParameters) kp.getPrivate();

        RSAKeyParameters rsakp = (RSAKeyParameters) kp.getPublic();

        KeyFactory fact = KeyFactory.getInstance("RSA", "BC");

        RSAPrivateCrtKeySpec prvKeySpecs = new RSAPrivateCrtKeySpec(rsapckp.getModulus(), rsapckp.getPublicExponent(), rsapckp.getExponent(), rsapckp.getP(), rsapckp.getQ(), rsapckp.getDP(), rsapckp.getDQ(), rsapckp.getQInv());

        PrivateKey privateKey = fact.generatePrivate(prvKeySpecs);

        RSAPublicKeySpec pubKeySpecs = new RSAPublicKeySpec(rsakp.getModulus(), rsakp.getExponent());
        PublicKey publicKey = fact.generatePublic(pubKeySpecs);

        KeyPair newkp = new KeyPair(publicKey, privateKey);
        // Export
        StringWriter writer = new StringWriter();
        JcaPEMWriter wr = new JcaPEMWriter(writer);
        wr.writeObject(newkp);
        wr.flush();
        wr.close();
        LOG.debug("Key generated for cache");
        return (writer.toString());

    }
}

==> open-xchange-guard_2.4.2.orig.tar.bz2: open-xchange-guard-2.4.2/com.openexchange.guard.keys/src/com/openexchange/guard/keys/storage/keyCache/PopulateRSA.java <==
/*
 *
 *    OPEN-XCHANGE legal information
 *
 *    All intellectual property rights in the Software are protected by
 *    international copyright laws.
 *
 *
 *    In some countries OX, OX Open-Xchange, open xchange and OXtender
 *    as well as the corresponding Logos OX Open-Xchange and OX are registered
 *    trademarks of the OX Software GmbH group of companies.
 *    The use of the Logos is not covered by the GNU General Public License.
 *    Instead, you are allowed to use these Logos according to the terms and
 *    conditions of the Creative Commons License, Version 2.5, Attribution,
 *    Non-commercial, ShareAlike, and the interpretation of the term
 *    Non-commercial applicable to the aforementioned license is published
 *    on the web site http://www.open-xchange.com/EN/legal/index.html.
 *
 *    Please make sure that third-party modules and libraries are used
 *    according to their respective licenses.
 *
 *    Any modifications to this package must retain all copyright notices
 *    of the original copyright holder(s) for the original code used.
 *
 *    After any such modifications, the original and derivative code shall remain
 *    under the copyright of the copyright holder(s) and/or original author(s)per
 *    the Attribution and Assignment Agreement that can be located at
 *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
 *    given Attribution for the derivative code and a license granting use.
 *
 *     Copyright (C) 2016-2020 OX Software GmbH
 *     Mail: info@open-xchange.com
 *
 *
 *     This program is free software; you can redistribute it and/or modify it
 *     under the terms of the GNU General Public License, Version 2 as published
 *     by the Free Software Foundation.
 *
 *     This program is distributed in the hope that it will be useful, but
 *     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 *     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 *     for more details.
 *
 *     You should have received a copy of the GNU General Public License along
 *     with this program; if not, write to the Free Software Foundation, Inc., 59
 *     Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

package com.openexchange.guard.keys.storage.keyCache;

import java.io.IOException;
import java.io.StringWriter;
import java.math.BigInteger;
import java.security.KeyFactory;
import java.security.KeyPair;
import java.security.NoSuchAlgorithmException;
import java.security.NoSuchProviderException;
import java.security.PrivateKey;
import java.security.PublicKey;
import java.security.SecureRandom;
import java.security.spec.InvalidKeySpecException;
import java.security.spec.RSAPrivateCrtKeySpec;
import java.security.spec.RSAPublicKeySpec;
import java.sql.SQLException;
import org.bouncycastle.crypto.AsymmetricCipherKeyPair;
import org.bouncycastle.crypto.generators.RSAKeyPairGenerator;
import org.bouncycastle.crypto.params.RSAKeyGenerationParameters;
import org.bouncycastle.crypto.params.RSAKeyParameters;
import org.bouncycastle.crypto.params.RSAPrivateCrtKeyParameters;
import org.bouncycastle.openssl.jcajce.JcaPEMWriter;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import com.openexchange.exception.OXException;
import com.openexchange.guard.cipher.GuardCipherAlgorithm;
import com.openexchange.guard.cipher.GuardCipherFactoryService;
import com.openexchange.guard.cipher.GuardCipherService;
import com.openexchange.guard.common.java.Strings;
import com.openexchange.guard.configuration.GuardConfigurationService;
import com.openexchange.guard.configuration.GuardProperty;
import com.openexchange.guard.keys.GuardMasterKeyService;
import com.openexchange.guard.keys.osgi.Services;
import com.openexchange.guard.keys.storage.KeyCacheStorage;

public class PopulateRSA implements Runnable {

    private static Logger LOG = LoggerFactory.getLogger(PopulateRSA.class);

    @Override
    public void run() {
        try {
            KeyCacheStorage keyCacheStorage = Services.getService(KeyCacheStorage.class);
            GuardConfigurationService configService = Services.getService(GuardConfigurationService.class);
            int count = keyCacheStorage.getCount();
            int rsaCacheCount = configService.getIntProperty(GuardProperty.rsaCacheCount);
            if (count < rsaCacheCount) {
                while (count < rsaCacheCount) {
                    count++;
                    String newkey = genKeyPair();
                    storekey(newkey);
                }
            }
        } catch (Exception e) {
            LOG.error("Unable to access key_cache table.  Check database permissions", e);
        }
    }

    /**
     * Store RSA key PEM for future use if keyasc param is not empty. Encrypt with Config.rpass
     *
     * @param keyAsc the ascii armored to to save
     * @throws SQLException due an error
     */
    private void storekey(String keyAsc) throws SQLException {
        if (Strings.isEmpty(keyAsc)) {
            LOG.warn("Unable to store key as provided ascii armored version is empty!");
            return;
        }

        try {
            //encrypting the key for storing it in the DB
            GuardCipherService cipherService = Services.getService(GuardCipherFactoryService.class).getCipherService(GuardCipherAlgorithm.AES_CBC);
            GuardMasterKeyService masterKeyService = Services.getService(GuardMasterKeyService.class);
            keyAsc = cipherService.encrypt(keyAsc, masterKeyService.getMasterKey().getRC(), "key");

            KeyCacheStorage keyCacheStorage = Services.getService(KeyCacheStorage.class);
            keyCacheStorage.insert(keyAsc);
        } catch (Exception ex) {
            LOG.error("Error while storing key", ex);
        }
    }

    /**
     * Generate RSA keypair and return as PEM
     *
     * @return
     * @throws IOException
     * @throws NoSuchAlgorithmException
     * @throws InvalidKeySpecException
     * @throws NoSuchProviderException
     * @throws OXException
     */
    private String genKeyPair() throws IOException, NoSuchAlgorithmException, InvalidKeySpecException, NoSuchProviderException, OXException {
        GuardConfigurationService configService = Services.getService(GuardConfigurationService.class);
        RSAKeyPairGenerator kpgen = new RSAKeyPairGenerator();
        kpgen.init(new RSAKeyGenerationParameters(BigInteger.valueOf(0x10001), new SecureRandom(), configService.getIntProperty(GuardProperty.rsaKeyLength), configService.getIntProperty(GuardProperty.rsaCertainty)));
        AsymmetricCipherKeyPair kp = kpgen.generateKeyPair();
        RSAPrivateCrtKeyParameters rsapckp = (RSAPrivateCrtKeyParameters) kp.getPrivate();

        RSAKeyParameters rsakp = (RSAKeyParameters) kp.getPublic();

        KeyFactory fact = KeyFactory.getInstance("RSA", "BC");

        RSAPrivateCrtKeySpec prvKeySpecs = new RSAPrivateCrtKeySpec(rsapckp.getModulus(), rsapckp.getPublicExponent(), rsapckp.getExponent(), rsapckp.getP(), rsapckp.getQ(), rsapckp.getDP(), rsapckp.getDQ(), rsapckp.getQInv());

        PrivateKey privateKey = fact.generatePrivate(prvKeySpecs);

        RSAPublicKeySpec pubKeySpecs = new RSAPublicKeySpec(rsakp.getModulus(), rsakp.getExponent());
        PublicKey publicKey = fact.generatePublic(pubKeySpecs);

        KeyPair newkp = new KeyPair(publicKey, privateKey);
        // Export
        StringWriter writer = new StringWriter();
        JcaPEMWriter wr = new JcaPEMWriter(writer);
        wr.writeObject(newkp);
        wr.flush();
        wr.close();
        LOG.debug("Key generated for cache");
        return (writer.toString());

    }
}
